// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/***************************************************************************

   QuotedString.h                                               SAS

   (c) 1998 European Space Agency - Written by Marco Beijersbergen

   $Id: QuotedString.h,v 1.7 2008/09/26 16:38:55 eojero Exp $

 **************************************************************************/

#ifndef QuotedString_h
#define QuotedString_h

#include <string>
#include <iostream>

class Quotes {

  public:

    Quotes(char o='"', char c='"') : open(o), close(c) {}

    char open, close;
};

class QuotedString : public std::string {

  public:

    QuotedString() {}

    QuotedString(const Quotes& q) : _quotes(q) {}
    QuotedString(char open, char close) : _quotes(open,close) {}

    QuotedString(const std::string& s) : std::string(s) {}
    QuotedString(const std::string& s, size_type pos, size_type n = npos)
	 : std::string(s,pos,n) {}
    QuotedString(const char *t, int len) : std::string(t,len) {}
    QuotedString(const char *t) : std::string(t) {}
    QuotedString(size_type n, char c) : std::string(n,c) {}

    const Quotes& quotes() const { return _quotes; }
    Quotes& quotes() { return _quotes; }
    void quotes(char open, char close) { _quotes = Quotes(open,close); }
    void quotes(const Quotes& q) { _quotes = q; }

    bool operator==(const QuotedString& s) const;
    bool operator==(const std::string& s) const;

  private:

    Quotes _quotes;
};

inline bool QuotedString::operator==(const QuotedString& s) const
{
  return (compare(s) == 0);
}
inline bool QuotedString::operator==(const std::string& s) const
{
  return (compare(QuotedString(s)) == 0);
}


inline std::istream& operator>>(std::istream& is, QuotedString& qs)
{
  char ch;

  if (is >> std::ws && is.get(ch)) {

    if (ch == qs.quotes().open) {
      qs = "";
      while (is.get(ch) && ch != qs.quotes().close)
        qs += ch;
    }
    else {
      is.unget();
      is >> (std::string&)qs;

      // Workaround for string bug: operator>> appends the trailing 0
      // from a stringstream.
      if (qs[qs.length()-1] == 0) qs.resize(qs.length()-1);
    }
  }
  return is;
}

inline std::ostream& operator<<(std::ostream& os, const QuotedString& s)
{
  if (s.empty() || s.find_first_of(" \t\n\r") != std::string::npos)
    os << s.quotes().open << (std::string&)s << s.quotes().close;
  else
    os << (std::string&)s;
  return os;
}

#endif






