// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Copyright 1999 (c) European Space Agency - written by Marco Beijersbergen

#ifndef Tracer_h
#define Tracer_h

#include <string>
#include <list>
#include <iostream>
#include <cstdlib>

class Tracer {

  public:

    enum Verbosity { Silent=0, Stack, Sparse, Verbose };

    Tracer(std::ostream& os = std::cerr);

    ~Tracer();

    void in(const std::string& mess);

    void out();

    void trace(bool enabled=true) { _verbosity = (enabled ? Verbose:Sparse); }

    bool tracing() const { return _verbosity >= Verbose; }

    void verbosity(Verbosity v) { _verbosity = v; }

    Verbosity verbosity() const { return _verbosity; }

  private:

    std::string indent() const { return std::string(2*_indent,' '); }

    std::ostream& _os;
    Verbosity _verbosity;
    int _indent;
    std::list< std::string > _stack;
};

extern Tracer tracer;

#ifdef TRACE

#define PROC            { if (tracer.verbosity()) tracer.in(__PRETTY_FUNCTION__); }
#define ENDPROC         { if (tracer.verbosity()) tracer.out(); }
#define RETURN          return (tracer.verbosity() ? tracer.out(),0:0 ),
#define ALTRETURNV(x)   { if (tracer.verbosity()) tracer.out(); return x; }
#define ALTRETURN       { if (tracer.verbosity()) tracer.out(); return; }

#define SAS_DEBUG(command)	{ if (tracer.verbosity() >= Tracer::Sparse) { command } }

#else

#define PROC
#define ENDPROC
#define RETURN          return
#define ALTRETURNV(x)   { return (x); }
#define ALTRETURN       { return; }

#define SAS_DEBUG(command)

#endif

#endif
