// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/****************************************************************************
 *
 *  testOptional.cc
 *
 *  Test program for Optional class
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: testOptional.cc,v 1.6 2008/09/26 16:38:55 eojero Exp $
 *
 *****************************************************************************/

#include "Optional.h"

#include <iostream>
#include <string>

#include <cassert>
#include <cmath>

//================================================================
//  Test program
//================================================================

int main()
{
     // Default constructor
    Optional<int> x;
    assert(!x.isDefined());

    // Copy constructor with undefined value
    Optional<int> a(x);
    assert(!a.isDefined());

    // Assignment to underlying type
    x = 33;
    assert(x.isDefined());
    assert(x == 33);
    assert(x.value() == 33);

    // Assignment to another Optional value
    Optional<int> c;
    c = x;
    assert(c.isDefined());
    assert(c == 33);

    // Construct from underling type
    Optional<int> y(42);
    assert(y.isDefined());
    assert(y == 42);

    // Default constructor with a class type
    Optional<std::string> s;
    assert(!s.isDefined());

    // Assignment to a class type
    s = std::string("hello");
    assert(s.isDefined());
    assert(std::string("hello") == s.value());

    // Conversion to underlying type
    double z = y + 3.0;
    assert(fabs(z-45) < 1e-6);

    return 0;
}
