// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/****************************************************************************
 *
 *  testStringFns.cc                           XMM Science Analysis Subsystem
 *
 *  Test program for StringFns class
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: testStringFns.cc,v 1.4 2008/09/26 16:38:55 eojero Exp $
 *
 *****************************************************************************/

#include "StringFns.h"

#include <iostream>
#include <string>
#include <list>

#include <cassert>


//================================================================
//  Test program
//================================================================

// Compare two string lists

bool compare_string_lists(std::list<std::string> a, std::list<std::string> b)
{
    if(a.size() != b.size())
        return false;

    bool ok = true;
    std::list<std::string>::const_iterator ita = a.begin();
    std::list<std::string>::const_iterator itb = a.begin();
    while(ok && ita != a.end() && itb != b.end())
        ok &= (*ita++ == *itb++);

    return ok;
}

int main()
{
    // Split a string into tokens

    std::list<std::string> ls1 = StringFns::splitString("ooxyaaapqor", "oa");
    assert(ls1.size() == 3);
    assert(ls1.front() == std::string("xy"));
    ls1.pop_front();
    assert(ls1.front() == std::string("pq"));
    ls1.pop_front();
    assert(ls1.front() == std::string("r"));

    // Split a quoted string into tokens

    bool ok1 = false;
    std::list<std::string> ls2 = StringFns::splitStringList("'a b' 'c d'", &ok1);
    assert(ok1);
    assert(ls2.size() == 2);
    assert(ls2.front() == std::string("a b"));
    ls2.pop_front();
    assert(ls2.front() == std::string("c d"));

    // Split a quoted string - mismatched quotes

    bool ok2 = false;
    std::list<std::string> ls3 = StringFns::splitStringList("'a b' 'c d", &ok2);
    assert(!ok2);

    // Parse an integer

    int n = 0;
    bool ok3 = StringFns::parseInt("123", n);
    assert(ok3);
    assert(n == 123);

   // Parse an integer - invalid string

    bool ok4 = StringFns::parseInt("12.3", n);
    assert(!ok4);
    assert(n == 123);               // Should be unchanged

    // Parse a double

    double x = 0;
    bool ok5 = StringFns::parseDouble("12.3e2", x);
    assert(ok5);
    assert(x == 1230);

   // Parse a double - invalid string

    bool ok6 = StringFns::parseDouble("12x", x);
    assert(!ok6);
    assert(x == 1230);              // Should be unchanged

    return 0;
}

