// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <iostream>
#include <string>
#include <cassert>

#include <Tracer.h>
#include <f90cxx.h>

F90MODPROCPROTO(void,amod,printstring)(void);

F90MODPROCPROTO(void,amod,printarray1)(const Dope<int,1>&);

F90MODPROCPROTO(void,amod,printarray2)(const Dope<int,2>&);

F90MODPROCPROTO(void,amod,callwitharray1)(void);

F90MODPROCPROTO(void,amod,callwitharray2)(void);

F90MODPROCPROTO(void,amod,callreplaced)(void);

F90MODPROCPROTO(void,amod,printtypes)(char&,short&,int&,float&,double&);

F90MODPROCPROTO(void,amod,callwithtypes)();

extern "C" void replaced_(void) { std::cout << "The proper replaced()"<<std::endl; }

F90PROCDEF(void,cxxfunction1)(Dope<int,1>& d)
{ 
  PROC
  std::clog << d;
#ifdef NAGf90Fortran
  assert(d.dim[0].extent == 4);
  assert(d.dim[0].lower == 1);
#else
  assert(d.dim[0]._ubound == 4);
  assert(d.dim[0].lower_bound == 1);
#endif
  const int *p = (int*)d.addr;
  assert(p[0] == 1);
  assert(p[1] == 2);
  assert(p[2] == 3);
  assert(p[3] == 4);
  ENDPROC
}

F90PROCDEF(void,cxxfunction2)(Dope<int,2>& d)
{
  PROC
  std::clog << d;
#ifdef NAGf90Fortran
  assert(d.dim[0].extent == 2);
  assert(d.dim[0].lower == 1);
  assert(d.dim[1].extent == 3);
  assert(d.dim[1].lower == 1);
#else
  assert(d.dim[0]._ubound == 2);
  assert(d.dim[0].lower_bound == 1);
  assert(d.dim[1]._ubound == 3);
  assert(d.dim[1].lower_bound == 1);
#endif
  int *p = (int*)d.addr;
  assert(p[0] == 11);
  assert(p[1] == 21);
  assert(p[2] == 12);
  assert(p[3] == 22);
  ENDPROC
}

F90PROCDEF(void,alltypes)(char& b, short& s, int& i, float& f, double& d)
{
  PROC
  assert(b == 100);
  assert(s == -1000);
  assert(i == 10000);
  assert(f == -1.23e4);
  assert(d == 1.23456e7);
  ENDPROC
}

F90PROCDEF(void,error)(char *code, char* mess, int codeLen, int messLen)
{
  PROC
  std::cerr << "Error " << std::string(code,codeLen) << ": "
       << std::string(mess,messLen) << std::endl;
  std::exit(1);
  ENDPROC
}

int main()
{
  PROC

// As in f90cxx.h, the f90_io_init and f90_io_finish are only known to NAG Fortran

#ifdef NAGf90Fortran
  f90init();
#endif

  int a1[4] = { 10, 20, 30, 40 };
  unsigned long ext1[1] = {4};
//  int a2[6] = { 110,120,210,220,310,320 };
  int a2[3][2] = { {110,120},{210,220},{310,320}};
  unsigned long ext2[2] = {3,2};

  char b = -101;
  short s = 1001;
  int i = -10001;
  float f = 2.34e5;
  double d = -2.3456e7;

  F90MODPROC(amod,printstring)();

  Dope<int,1> dope1(a1,ext1);
  std::clog << dope1;

  F90MODPROC(amod,printarray1)(dope1);

  Dope<int,2> dope2((int*)a2,ext2);
  std::clog << dope2;
  F90MODPROC(amod,printarray2)(dope2);

  F90MODPROC(amod,callwitharray1)();
  F90MODPROC(amod,callwitharray2)();
  F90MODPROC(amod,callreplaced)();
  F90MODPROC(amod,printtypes)(b,s,i,f,d);
  F90MODPROC(amod,callwithtypes)();

#ifdef NAGf90Fortran
  f90finish();
#endif

  RETURN 0;
}
